@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ConfigDefs.pl 3833 2007-01-30 18:20:13Z sysjkf $
+#   $Id: ConfigDefs.pl 4345 2008-03-16 22:27:07Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -52,24 +52,36 @@ __DATA__
 
 addenvfrom			= AddEnvelopeFromHeader
 addenvto			= AddEnvelopeToHeader
+addmshmac			= AddWatermark
 AllowObjectTags			= AllowObjectCodebaseTags
 AllowExternal			= AllowExternalMessageBodies
 AllowPartial			= AllowPartialMessages
 allowpasszips			= AllowPasswordProtectedArchives
 AllowWebBugTags			= AllowWebBugs
+attachimage			= AttachImageToSignature
+attachimagename			= SignatureImageFilename
+attachimageinternalname		= SignatureImageImgFilename
+attachimagetohtmlonly		= AttachImageToHTMLMessageOnly
 AttachmentCharset		= AttachmentEncodingCharset
+AttachZipMinSize		= AttachmentsMinTotalSizeToZip
+AttachZipName			= AttachmentsZipFilename
+AttachZipIgnore			= AttachmentExtensionsNotToZip
 bayesrebuild			= RebuildBayesEvery
 bayeswait			= WaitDuringBayesRebuild
 blacklistedishigh		= definitespamishighscoring
 BlockEncrypted			= BlockEncryptedMessages
 BlockUnencrypted		= BlockUnencryptedMessages
 cachetiming			= SpamAssassinCacheTimings
+checkmshmac			= CheckWatermarksWithNoSender
+checkmshmacskip			= CheckWatermarksToSkipSpamChecks
+checkppafilenames		= CheckFilenamesInPasswordProtectedArchives
 CheckSAIfOnSpamList		= checkspamassassinifonspamlist
 children			= maxchildren
 clamavmaxreclevel		= clamavmodulemaximumrecursionlevel
 clamavmaxfiles			= clamavmodulemaximumfiles
 clamavmaxfilesize		= clamavmodulemaximumfilesize
 clamavmaxratio			= clamavmodulemaximumcompressionratio
+clamavspam			= ClamAVFullMessageScan
 clamwatchfiles			= monitorsforclamavupdates
 cleanheader			= cleanheadervalue
 contentmodifysubject		= contentmodifysubject
@@ -90,6 +102,7 @@ disinfectedreporttext		= disinfectedreport
 envfromheader			= EnvelopeFromHeader
 envtoheader			= EnvelopeToHeader
 findphishing			= FindPhishingFraud
+fprotd6port			= FpscandPort
 gsscanner			= UseCustomSpamScanner
 gstimeout			= CustomSpamScannerTimeout
 gstimeoutlen			= CustomSpamScannertimeouthistory
@@ -119,6 +132,8 @@ lastlookup			= alwayslookeduplast
 listsascores                    = includescoresinspamassassinreport
 loghtmltags			= logdangeroushtmltags
 logfacility			= syslogfacility
+logsaactions			= logspamassassinruleactions
+logsock				= syslogsockettype
 lookforuu			= finduuencodedfiles
 maxattachmentsize		= maximumattachmentsize
 maxdirtybytes			= maxunsafebytesperscan
@@ -132,6 +147,10 @@ maxunscannedmessages		= maxunscannedmessagesperscan
 maxzipdepth			= maximumarchivedepth
 minattachmentsize		= minimumattachmentsize
 minstars			= minimumstarsifonspamlist
+mshmac				= WatermarkSecret
+mshmacheader			= WatermarkHeader
+mshmacnull			= TreatInvalidWatermarksWithNoSenderAsSpam
+mshmacvalid			= WatermarkLifetime
 namemodifysubject		= filenamemodifysubject
 namesubjecttext			= filenamesubjecttext
 noisyviruses			= nonforgingviruses
@@ -139,6 +158,7 @@ normalrbls                      = spamliststobespam
 nosenderprecedence		= nevernotifysendersofprecedence
 noticefullheaders		= noticesincludefullheaders
 noticerecipient			= noticesto
+phishingblacklist		= phishingbadsitesfile
 phishinghighlight		= highlightphishingfraud
 phishingnumbers			= alsofindnumericphishing
 phishingsubjecttag		= phishingsubjecttext
@@ -150,6 +170,8 @@ quarantinegroup			= quarantinegroup
 quarantineperms			= quarantinepermissions
 rbltimeoutlen			= spamlisttimeoutshistory
 usesacache			= cachespamassassinresults
+saactions			= spamassassinruleactions
+sadecodebins			= IncludeBinaryAttachmentsInSpamAssassin
 satimeoutlen			= spamassassintimeoutshistory
 removeheaders			= removetheseheaders
 replacetnef			= usetnefcontents
@@ -160,6 +182,7 @@ scanmail			= scanmessages
 scoreformat			= spamscorenumberformat
 secondlevellist                 = countrysubdomainslist
 sendercontentreport		= senderbadcontentreport
+# JKF 19/12/2007 senderpasswordreport		= senderbadpasswordprotectedarchivereport
 senderfilenamereport		= senderbadfilenamereport
 senderrblspamreport		= senderspamlistreport
 sendersaspamreport		= senderspamassassinreport
@@ -177,6 +200,7 @@ spamdetail			= detailedspamreport
 mcpmodifysubject		= mcpmodifysubject
 sizemodifysubject		= sizemodifysubject
 sizesubjecttext			= sizesubjecttext
+spamassassintempdir		= spamassassintemporarydir
 spammodifysubject		= spammodifysubject
 spamscorenotstars		= spamscorenumberinsteadofstars
 spamstars			= spamscore
@@ -196,6 +220,7 @@ mcpdetail			= detailedmcpreport
 mcplistsascores			= includescoresinmcpreport
 mcpreqspamassassinscore		= mcprequiredspamassassinscore
 mcpwhitelist			= isdefinitelynotmcp
+syntaxcheck			= automaticsyntaxcheck
 unscannedheader			= unscannedheadervalue
 usedefaultswithmanyrecips       = usedefaultruleswithmultiplerecipients
 tagphishingsubject		= phishingmodifysubject
@@ -207,7 +232,9 @@ warnvirussenders		= notifysendersofviruses
 warnnamesenders			= notifysendersofblockedfilenamesorfiletypes
 warnsizesenders                 = notifysendersofblockedsizeattachments
 warnothersenders		= notifysendersofotherblockedcontent
+# JKF 19/12/2007 warnpasswordsenders		= notifysendersofblockedpasswordprotectedarchives
 webbugurl			= webbugreplacement
+webbugblacklist			= knownwebbugservers
 webbugwhitelist			= ignoredwebbugfilenames
 whitelistmaxrecips		= ignorespamwhitelistifrecipientsexceed
 workuser			= incomingworkuser
@@ -226,10 +253,12 @@ workperms			= incomingworkpermissions
 #
 [Simple,YesNo]
 bayeswait		0	no	0	yes	1
+clamavspam		0	no	0	yes	1
 debug			0	no	0	yes	1
 debugspamassassin	0	no	0	yes	1
 deliverinbackground	1	no	0	yes	1
 lognonspam		0	no	0	yes	1
+logsaactions		0	no	0	yes	1
 logsilentviruses	0	no	0	yes	1
 logspam			0	no	0	yes	1
 logspeed		0	no	0	yes	1
@@ -240,6 +269,7 @@ showscanner		1	no	0	yes	1
 spamassassinautowhitelist 1	no	0	yes	1
 spliteximspool		0	no	0	yes	1
 storeentireasdfqf	0	no	0	yes	1
+syntaxcheck		1	no	0	yes	1
 usedefaultswithmanyrecips	0	no	0	yes	1
 #virusbeforespammcp	0	no	0	yes	1
 
@@ -251,6 +281,7 @@ lockfiledir		/tmp
 # Check the first word of these for file existence
 [Simple,File]
 PhishingWhitelist	/etc/MailScanner/phishing.safe.sites.conf
+PhishingBlacklist	/etc/MailScanner/phishing.bad.sites.conf
 pidfile			/var/run/MailScanner.pid
 SecondLevelList         /etc/MailScanner/country.domains.conf
 #spamassassinprefsfile	/etc/MailScanner/spam.assassin.prefs.conf
@@ -266,8 +297,10 @@ clamavmaxreclevel               8
 clamavmaxfiles                  1000
 clamavmaxfilesize               10000000
 clamavmaxratio                  250
+ClamdPort 3310
 CriticalQueueSize		800
 FileTimeout			20
+fprotd6port			10200
 GSTimeout			20
 GSTimeoutLen			20
 GunzipTimeout			50
@@ -312,6 +345,7 @@ LDAPsite
 # LockType *must not* have a static default
 LockType
 LogFacility		mail
+LogSock		
 MailScannerVersionNumber	1.0.0
 MaxSpamAssassinSize		30000
 MinimumCodeStatus	supported
@@ -326,6 +360,7 @@ SAVIWatchFiles		/usr/local/Sophos/ide/*.zip
 SophosAllowedErrors	
 sophoside		
 sophoslib		
+spamassassintempdir	/var/spool/MailScanner/incoming/SpamAssassin-Temp
 SpamAssassinUserStateDir
 SpamAssassinSiteRulesDir
 SpamAssassinLocalRulesDir	
@@ -380,6 +415,7 @@ ListSAScores		1	no	0	yes	1
 LogHTMLTags		0	no	0	yes	1
 LogPermittedFilenames	0	no	0	yes	1
 LogPermittedFiletypes	0	no	0	yes	1
+LogPermittedFileMimetypes	0	no	0	yes	1
 LookForUU		0	no	0	yes	1
 MultipleHeaders		append	append	append	replace	replace	add	add
 NameModifySubject	start	no	0	yes	1	start	start	end	end
@@ -412,6 +448,7 @@ WarnVirusSenders	0	no	0	yes	1
 WarnNameSenders		1	no	0	yes	1
 WarnSizeSenders		0	no	0	yes	1
 WarnOtherSenders	1	no	0	yes	1
+# JKF 19/12/2007 WarnPasswordSenders    1       no      0       yes     1
 
 [First,File]
 DeletedContentMessage	/etc/MailScanner/reports/en/deleted.content.message.txt
@@ -429,6 +466,7 @@ recipientmcpreport	/etc/MailScanner/reports/en/recipient.mcp.report.txt
 recipientspamreport	/etc/MailScanner/reports/en/recipient.spam.report.txt
 rejectionreport		/etc/MailScanner/reports/en/message.rejection.report.txt
 sendercontentreport 	/etc/MailScanner/reports/en/sender.content.report.txt
+# JKF 19/12/2007 senderpasswordreport   /etc/MailScanner/reports/en/sender.password.report.txt
 sendererrorreport 	/etc/MailScanner/reports/en/sender.error.report.txt
 senderfilenamereport	/etc/MailScanner/reports/en/sender.filename.report.txt
 SenderRBLSpamReport	/etc/MailScanner/reports/en/sender.spam.rbl.report.txt
@@ -451,6 +489,7 @@ PublicKeyArchiveDir		#/var/spool/MailScanner/keys
 quarantinedir			/var/spool/MailScanner/quarantine
 
 [First,Number]
+AttachZipMinSize		100000
 HighRBLs			3
 HighSpamAssassinScore		10
 MaxAttachmentSize		-1
@@ -463,12 +502,16 @@ MCPHighSpamAssassinScore	10
 MCPReqSpamAssassinScore		1
 MinAttachmentSize		-1
 MinStars			0
+mshmacvalid			604800
 NormalRBLs			1
 ReqSpamAssassinScore		6
 
 [First,Other]
 AttachmentCharset		ISO-8859-1
+attachimageinternalname
+attachimagename
 AttachmentWarningFilename	VirusWarning.txt
+AttachZipName			MessageAttachments.zip
 cleanheader			Found to be clean
 ContentSubjectText		{Dangerous Content?}
 dirtyheader			Found to be infected
@@ -483,6 +526,8 @@ InfoHeader
 InfoValue			Please contact the ISP for more information
 LocalPostmaster			postmaster
 MailHeader			X-MailScanner:
+mshmac				Watermark-secret
+mshmacheader			MailScanner-NULL-Check:
 NameSubjectText			{Filename?}
 NoticesFrom			MailScanner
 NoticeSignature			-- \nMailScanner\nEmail Virus Scanner\nwww.mailscanner.info
@@ -510,14 +555,21 @@ SizeSubjectText		{Size}
 [All,YesNo]
 AddEnvFrom		1	no	0	yes	1
 AddEnvTo		0	no	0	yes	1
+addmshmac		1	no	0	yes	1
 AllowIFrameTags		convert	no	0	yes	1	disarm	convert
 AllowFormTags		convert	no	0	yes	1	disarm	convert
 AllowObjectTags		convert	no	0	yes	1	disarm	convert
 AllowScriptTags		convert	no	0	yes	1	disarm	convert
 AllowPassZips		0	no	0	yes	1
 AllowWebBugTags		convert	no	0	yes	1	disarm	convert
+attachimage		0	no	0	yes	1
+attachimagetohtmlonly	1	no	0	yes	1
 BlockEncrypted		0	no	0	yes	1
 BlockUnencrypted	0	no	0	yes	1
+checkppafilenames	1	no	0	yes	1
+checkmshmac		1	no	0	yes	1
+checkmshmacskip		1	no	0	yes	1
+ClamdUseThreads		0	no	0	yes	1
 DangerScan		1	no	0	yes	1
 DeliverCleanedMessages	1	no	0	yes	1
 FindPhishing		1	no	0	yes	1
@@ -531,13 +583,16 @@ QuarantineSilent	0	no	0	yes	1
 QuarantineWholeMessage	0	no	0	yes	1
 RejectMessage		0	no	0	yes	1
 ReplaceTNEF		2	no	0	add	1	replace	2
+sadecodebins		0	no	0	yes	1
 ScanMail		1	no	0	yes	1
 SpamStars		1	no	0	yes	1
 StrictPhishing          1       no      0       yes     1
 TagPhishingSubject	0	no	0 	yes	1	start	start	end	end
 MCPUseSpamAssassin	1	no	0	yes	1
 UseSpamAssassin		1	no	0	yes	1
+UseWatermarking		1	no	0	yes	1
 VirusScan		1	no	0	yes	1
+ZipAttachments		0	no	0	yes	1
 
 [All,File]
 #FilenameRules		/etc/MailScanner/filename.rules.conf
@@ -546,16 +601,25 @@ VirusScan		1	no	0	yes	1
 # This is the other stuff that came up in the search that I haven't
 # figured out what to do with yet...
 ArchiveMail
+AttachZipIgnore			.zip .rar .gz .tgz .mpg .mpe .mpeg .mp3 .rpm
+ClamdLockFile
+ClamdSocket 127.0.0.1
 FilenameRules		
 FiletypeRules		
+mshmacnull			spam
 NoisyViruses			Joke/ OF97/ WM97/ W97M/ eicar
 NoSenderPrecedence		list bulk
 NoticeRecipient			postmaster
 RemoveHeaders			X-Mozilla-Status: X-Mozilla-Status2:
 SilentViruses			HTML-IFrame All-Viruses
 SpamDomainList			
+webbugblacklist
 webbugwhitelist
 allowfilenames
+denyfilemimetypes
 denyfiletypes
+allowfilemimetypes
 allowfiletypes
 denyfilenames
+saactions
+
