@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm 3832 2007-01-30 18:01:19Z sysjkf $
+#   $Id: MessageBatch.pm 4295 2008-02-14 20:11:06Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,11 +41,12 @@ no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
 use Time::HiRes qw ( time );
+use POSIX;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3832 $, 10;
+$VERSION = substr q$Revision: 4295 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -71,6 +72,7 @@ my $initialised = 0;
 #       are about to remove it from the inqueue.
 sub new {
   my $type = shift;
+  my $lint = shift;
   my $this = {};
 
   #print STDERR "In new MessageBatch\n";
@@ -78,7 +80,13 @@ sub new {
   MailScanner::Log::DieLog("Tried to create a MessageBatch without calling initglobals")
     unless $initialised;
 
-  $global::MS->{mta}->CreateBatch($this);
+  if ($lint eq 'lint') {
+    # Fake a batch containing the Eicar message
+    bless $this, $type;
+    $this->CreateEicarBatch();
+  } else {
+    $global::MS->{mta}->CreateBatch($this);
+  }
 
   $this->{starttime} = time;
 
@@ -124,12 +132,7 @@ sub EndBatch {
 sub StartTiming {
   my $this = shift;
   my($varprefix, $usertext) = @_;
-
-  my $command = "";
-  my $now = time;
-
-  $command = '$this->{' . $varprefix . '_starttime} = $now;';
-  eval $command;
+  $this->{$varprefix . '_starttime'} = time;
 }
 
 # Stop the timing for a section of the main code
@@ -138,28 +141,23 @@ sub StopTiming {
   my $this = shift;
   my($varprefix, $usertext) = @_;
 
-  my $command = "";
   my $now = time;
   my $totaltime;
   my $speed;
 
   # totaltime = now - starttime
-  $command = '$totaltime = $now - $this->{' . $varprefix . '_starttime};';
-  eval $command;
+  $totaltime = $now - $this->{$varprefix . '_starttime'};
 
   # endtime = now
-  $command = '$this->{' . $varprefix . '_endtime} = $now;';
-  eval $command;
+  $this->{$varprefix . '_endtime'} = $now;
   # totaltime = totaltime
-  $command = '$this->{' . $varprefix . '_totaltime} = $totaltime;';
-  eval $command;
+  $this->{$varprefix . '_totaltime'} = $totaltime;
   $totaltime = 1 unless $totaltime > 0; # Minimum of 1 second
   # speed = bytes / totaltime
   $speed = ($this->{totalbytes}*1.0) / ($totaltime*1.0) if $totaltime > 0;
   $speed = 0 if $speed > 1_000_000 || $speed < 0;
   # bytespersecond = speed
-  $command = '$this->{' . $varprefix . '_bytespersecond} = $speed;';
-  eval $command;
+  $this->{$varprefix . '_bytespersecond'} = $speed;
 
   MailScanner::Log::InfoLog("%s completed at %d bytes per second",
                             $usertext, $speed)
@@ -251,11 +249,19 @@ sub SpamChecks {
   MailScanner::SA::CheckForCacheExpire();
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{scanmail} =~ /^[0\s]+$/;
+    next if !$message->{scanmail};
     next if $message->{deleted};
     next unless MailScanner::Config::Value('spamchecks', $message) =~ /1/;
  
     #print STDERR "Spam checks for $id\n";
+
+    # Tell SpamAssassin to apply rules to binary attachments
+    # This relies on a patch to Util.pm in SpamAssassin, fetch the patch from
+    # http://www.mailscanner.info/mcp.html#patches
+    $MailScanner::MCP::SA_Decode_Binaries = 0;
+    $MailScanner::MCP::SA_Decode_Binaries = 1
+      if MailScanner::Config::Value('sadecodebins', $message) =~ /1/;
+
     $counter += $message->IsSpam();
 
     if (!MailScanner::Config::Value('spamdetail', $message)) {
@@ -265,6 +271,7 @@ sub SpamChecks {
   }
   MailScanner::Log::NoticeLog("Spam Checks: Found $counter spam messages")
     if $counter>0;
+  $MailScanner::MCP::SA_Decode_Binaries = 0; # Reset 'are we in MCP' flag
   #print STDERR "$counter messages were spam\n";
 }
 
@@ -283,7 +290,7 @@ sub HandleSpam {
     #print STDERR "Deleted = " . $message->{deleted} . ", isspam = " . $message->{isspam} . ", scanmail = " . $message->{scanmail} . ", spamwhitelisted = " . $message->{spamwhitelisted} . "\n";
     next if  $message->{deleted} ||
             !$message->{isspam}  ||
-             $message->{scanmail} =~ /^[0\s]+$/ ||
+            !$message->{scanmail} ||
              $message->{spamwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -305,7 +312,7 @@ sub HandleHam {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
-            $message->{scanmail} =~ /^[0\s]+$/ ||
+           !$message->{scanmail} ||
             $message->{isspam};
 
     #print STDERR "Ham checks for $id\n";
@@ -344,9 +351,12 @@ sub MCPChecks {
   MailScanner::Log::InfoLog("MCP Checks: Starting")
     if MailScanner::Config::Value('logmcp');
 
+  # Tell SpamAssassin to apply rules to binary attachments
+  $MailScanner::MCP::SA_Decode_Binaries = 1;
+
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
-    next if $message->{scanmail} =~ /^[0\s]+$/;
+    next if !$message->{scanmail};
     next unless MailScanner::Config::Value('mcpchecks', $message);
  
     #print STDERR "Spam checks for $id\n";
@@ -361,6 +371,9 @@ sub MCPChecks {
   MailScanner::Log::NoticeLog("MCP Checks: Found $counter MCP messages")
     if $counter>0;
   #print STDERR "$counter messages were spam\n";
+
+  # Tell SpamAssassin to return to normal behaviour
+  $MailScanner::MCP::SA_Decode_Binaries = 0;
 }
 
 
@@ -377,7 +390,7 @@ sub HandleMCP {
     # Skip deleted and non-spam messages
     next if  $message->{deleted} ||
             !$message->{ismcp}   ||
-             $message->{scanmail} =~ /^[0\s]+$/ ||
+            !$message->{scanmail} ||
              $message->{mcpwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -399,7 +412,7 @@ sub HandleNonMCP {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
-            $message->{scanmail} =~ /^[0\s]+$/ ||
+           !$message->{scanmail} ||
             $message->{ismcp};
 
     #print STDERR "Ham checks for $id\n";
@@ -422,7 +435,7 @@ sub DeliverUnscanned {
     next if $message->{deleted} || $message->{dontdeliver};
 
     # This is for mail we don't want to touch at all
-    if ($message->{scanmail} =~ /^[0\s]+$/) {
+    if (!$message->{scanmail}) {
       $OutQ = MailScanner::Config::Value('outqueuedir', $message);
       $message->DeliverUntouched($OutQ);
       $message->{deleted} = 1; # This marks it for purging from disk
@@ -471,7 +484,7 @@ sub Explode {
   1 until waitpid(-1, WNOHANG) == -1;
 
   #print STDERR "About to explode messages\n";
-  print STDERR "Ignore errors about failing to find EOCD signature\n";
+  #print STDERR "Ignore errors about failing to find EOCD signature\n";
   umask $global::MS->{work}->{fileumask};
   while(($key, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
@@ -646,6 +659,19 @@ sub Clean {
   }
 }
 
+# Zip up all the attachments in the messages, to save space on the
+# mail servers.
+sub ZipAttachments {
+  my $this = shift;
+
+  my($id, $message);
+
+  while(($id, $message) = each %{$this->{messages}}) {
+    next if $message->{deleted} || $message->{dontdeliver};
+    #print STDERR "\nZipping attachments, message $id\n";
+    $message->ZipAttachments();
+  }
+}
 
 # Combine the virus and other reports and types for all the messages.
 # Might change this to do it at source later.
@@ -866,8 +892,9 @@ sub WarnSenders {
                   MailScanner::Config::Value('warnsizesenders', $message);
     $reasons++ if $message->{otherinfected} &&
                   MailScanner::Config::Value('warnothersenders', $message);
-    #$reasons++ if ($message->{otherinfected} || $message->{sizeinfected}) &&
-    #              MailScanner::Config::Value('warnothersenders', $message);
+    # JKF 19/12/2007
+    # $reasons++ if $message->{passwordinfected} &&
+    #               MailScanner::Config::Value('warnpasswordsenders', $message);
     next if $reasons==0;
 
     $message->WarnSender();
@@ -911,6 +938,7 @@ sub WarnLocalPostmaster {
       $reasons{filenameinfected}  = 1 if $message->{nameinfected};
       $reasons{otherinfected}     = 1 if $message->{otherinfected};
       $reasons{sizeinfected}      = 1 if $message->{sizeinfected};
+      # JKF 19/12/2007 $reasons{passwordinfected}  = 1 if $message->{passwordinfected};
       $reasons{passwordprotected} = 1 if $message->{passwordprotected};
       foreach $thisreason (sort keys %reasons) {
         $reasons .= " : " if $reasons ne "";
@@ -1117,5 +1145,64 @@ sub LastLookup {
   MailScanner::Config::Value('lastafterbatch', $this);
 }
 
+sub CreateEicarBatch {
+  my $batch = shift;
+
+  #print STDERR "Creating EICAR batch\n";
+  my $headerfileumask = $global::MS->{work}->{fileumask};
+
+  $batch->{messages} = {};
+  $batch->{totalmessages} = 1;
+  umask $headerfileumask;
+  # Create and write the header file
+  # Message number = 1
+  # Path = irrelevant as we're not actually reading anything
+  # It's a fake that we simulate ==> 1
+  my $newmessage = MailScanner::Message->new(1, '/tmp', 1);
+  @{$newmessage->{headers}} = ();
+  @{$newmessage->{to}} = ();
+  @{$newmessage->{touser}} = ();
+  @{$newmessage->{todomain}} = ();
+  $newmessage->{size} = 200;
+  $newmessage->{from} = 'sender@example.com';
+  $newmessage->{fromuser} = 'sender';
+  $newmessage->{fromdomain} = 'example.com';
+  push @{$newmessage->{to}}, 'recip@example.com';
+  push @{$newmessage->{touser}}, 'recip';
+  push @{$newmessage->{todomain}}, 'example.com';
+  $newmessage->{clientip} = '10.1.1.1';
+  $newmessage->{subject} = 'Virus Scanner Test Message';
+  push @{$newmessage->{headers}}, ('From: <sender@example.com>',
+                                   'To: <recip@example.com>',
+                                   'Subject: Virus Scanner Test Message',
+                                   'Mime-Version: 1.0',
+                                   'Content-Type: application/octet-stream; name="eicar.com"',
+                                   'Content-Transfer-Encoding: base64',
+'Content-Disposition: attachment; filename="eicar.com"'
+  );
+
+  $newmessage->{virusinfected} = 0;
+  $newmessage->{stillwarn} = 0;
+  $newmessage->{scanmail} = 1;
+
+  # Write out the .header file
+  $newmessage->WriteHeaderFile();
+
+  # Create a file of the body
+  my $fh = new FileHandle(">/tmp/MSLint.body.$$");
+  print $fh <<EOFLint;
+WDVPIVAlQEFQWzRcUFpYNTQoUF4pN0NDKTd9JEVJQ0FSLVNUQU5EQVJELUFOVElWSVJVUy1URVNU
+LUZJTEUhJEgrSCo=
+EOFLint
+  $fh->close();
+  $newmessage->{store}->{dpath} = "/tmp/MSLint.body.$$";
+
+  # Add it to the batch
+  $batch->{messages}{"1"} = $newmessage;
+  $newmessage->NeedsScanning(1);
+
+  umask 0077;
+}
+
 1;
 
