@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Postfix.pm 3830 2007-01-30 09:32:49Z sysjkf $
+#   $Id: Postfix.pm 4212 2007-11-20 19:59:26Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -37,11 +37,12 @@ use strict 'refs';
 no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
+use Encode;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3830 $, 10;
+$VERSION = substr q$Revision: 4212 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -288,8 +289,13 @@ sub new {
     # ORIGFound stuff courtesy of Juan Pablo Abuyeres 23/7/2006
     # Should improve handling of virtual domains.
     my($ORIGFound, $TOFound, $FROMFound, $IPFound, $TIMEFound);
-    my($ErrorFound, $ERecordFound, $rectype, $recdata);
+    my($ErrorFound, $ERecordFound, $rectype, $recdata, $mtime);
     my $InSubject = 0; # Are we adding continuation subject lines?
+    my $pRecordsFound = 0; # p record spin-through of body? //Glenn
+    my $OriginalPos = 0; # p record jumpoff point //Glenn
+    my $MaxpRecPos = 0; # Max position where a p record might occur.
+    my @npos = (); # save all p record positions, although we only
+                   # look at/use the first and last.
 
     #print STDERR "In ReadQf\n";
     #$message->{store}->print();
@@ -299,6 +305,10 @@ sub new {
     # Just in case we get a message with no headers at all
     @{$message->{headers}} = ();
 
+    # seek to end of file and save position, to make sure p records don't
+    # try go past this point. 17 is the record size for a p record.
+    seek $RQf, -17, 2;
+    $MaxpRecPos = tell $RQf;
 
     # Seek to the start of the file in case anyone read the file
     # between me opening it and locking it.
@@ -389,6 +399,57 @@ sub new {
         $message->{postfixrecips}{lc("$recdata")} = 1;
         $TOFound = 1;
         #print STDERR "Pre O Recip $recdata\n";
+      } elsif ($rectype eq 'p') {
+	# //Glenn 2007-01-16
+	# Handle p records (GOTO like in-place edit thing) by reading
+	# the pointed to data into the main message object, and
+	# silently discarding the actual p record.
+	# This p record should only point to added recipients, or
+	# moved records of the same type already handled in this
+	# segment, so lets just store the jumpoff point and loop.
+	# When we hit the next p record it should be the "jump back to
+        # original pos" one, or another forward p record... so we'll
+	# check that and act accordingly..
+        $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
+	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
+	if ($recdata+0 > $MaxpRecPos) {
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    $ErrorFound = 1;
+	    last;
+	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
+	if ($OriginalPos == 0) {
+	   # Jump to after E record and commence
+	   $OriginalPos = tell $RQf;
+	   seek $RQf, $recdata+0, 0; # jump. we should check this works.
+	   next;
+	} else {
+	  # We're at the return point, or moving even furtehr away...
+	  if ($recdata+0 < $OriginalPos) {
+	     # Error exit, carp or whatever. Probably should do the
+	     # nice exit Jules does above. For now, warn and use
+	     # OriginalPos
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
+	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0; # We're home again... Reset to prepare for
+			       # any new warps
+	     @npos = ();
+	  } else {
+	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
+	    if ($recdata+0 < $npos[$#npos-1]) {
+	       if ($recdata+0 >= $npos[0]) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $ErrorFound = 1;
+		  $OriginalPos = 0; # Reset to not fool next segment loop
+	          @npos = ();
+		  last;
+	       }
+	       $OriginalPos = 0; # We're home again... Reset to prepare for
+		                 # any new warps
+	       @npos = ();
+	    }
+	  }
+	}
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
@@ -407,6 +468,60 @@ sub new {
         last;
       }
       last if $rectype eq 'N' && $recdata eq "";
+      if ($rectype eq 'p') {
+	 # //Glenn 2007-01-16
+	 # Handle p records (GOTO like in-place edit thing) by reading
+	 # the pointed to data into the main message object, and
+	 # silently discarding the actual p record.
+	 # This p record should only point to added headers, or
+	 # moved records of the same type already handled in this
+	 # segment, so lets just store the jumpoff point and loop.
+	 # When we hit the next p record it should be the "jump back to
+         # original pos" one, or another forward p record... so we'll
+	 # check that and act accordingly..
+         $pRecordsFound = 1; # If we find a p record we need remember this to handle body ...
+	 next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
+	 if ($recdata+0 > $MaxpRecPos) {
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    $ErrorFound = 1;
+	    last;
+	 }
+	 push @npos, $recdata+0; # save jumpto position, for loop detection.
+	 if ($OriginalPos == 0) {
+	    # Jump to after E record and commence
+	    $OriginalPos = tell $RQf;
+	    seek $RQf, $recdata+0, 0; # jump. we should check this works.
+#	    next;
+	 } else {
+	   # We're at the return point, or moving even furtehr away...
+	   if ($recdata+0 < $OriginalPos) {
+	      # Error exit, carp or whatever. Probably should do the
+	      # nice exit Jules does above. For now, warn and use
+	      # OriginalPos
+	      MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
+	      seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	      $OriginalPos = 0; # We're home again... Reset to prepare for
+			        # any new warps
+	      @npos = ();
+	   } else {
+	     seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
+	     if ($recdata+0 < $npos[$#npos-1]) {
+	        if ($recdata+0 >= $npos[0]) {
+        	   MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		   $ErrorFound = 1;
+	           $OriginalPos = 0; # Reset to not fool next segment loop...
+	           @npos = ();
+		   last;
+	        }
+	        $OriginalPos = 0; # We're home again... Reset to prepare for
+		                  # any new warps
+	        @npos = ();
+	     }
+	   }
+	 }
+	 next; # done, don't add a spurious "converted p to N" record.
+      }
+      next if ($rectype eq 'w'); # Skip deleted (w) records ... Else they will be transformed to Normal (N) records.
       push @{$message->{headers}}, $recdata; # Headers have no leading N
       if ($recdata =~ /^Subject:\s*(\S.*)?$/i) {
         $message->{subject} = $1;
@@ -441,7 +556,89 @@ sub new {
 
     # We are now at the end of the headers. Jump straight to the metadata
     # after the message.
-    seek $RQf, $MsgContSize+$DataOffset, 0;
+#    seek $RQf, $MsgContSize+$DataOffset, 0;
+
+# Inelegant, but working. Instead of an efficient seek, we spinn through to
+# after X record. Unless we don't have a body to spin through. Also skip
+# the spin if we don't have any p records already (to not punish the normal
+# case).
+    if (!$message->{nobody} && $pRecordsFound) {
+      while(($rectype, $recdata) = ReadRecord($RQf)) {
+        #print STDERR "Metadata type $rectype data \"$recdata\"\n";
+        if (!defined($rectype) or $rectype eq 'X') {
+	  $ErrorFound = 1 if(!defined($rectype));
+          last;
+        }
+        if ($rectype eq 'p') {
+	  # //Glenn 2007-01-16
+	  # Handle p records (GOTO like in-place edit thing) by reading
+	  # the pointed to data into the main message object, and
+	  # silently discarding the actual p record.
+	  # This p record should only point to a new body record, or
+	  # moved records of the same type already handled in this
+	  # segment, so lets just store the jumpoff point and loop.
+	  # When we hit the next p record it should be the "jump back to
+          # original pos" one, or another forward p record... so we'll
+	  # check that and act accordingly..
+	  next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
+	  if ($recdata+0 > $MaxpRecPos) {
+	      MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	      $ErrorFound = 1;
+	      last;
+	  }
+	  push @npos, $recdata+0; # save jumpto position, for loop detection.
+	  if ($OriginalPos == 0) {
+	     # Jump to after E record and commence
+	     $OriginalPos = tell $RQf;
+	     seek $RQf, $recdata+0, 0; # jump. we should check this works.
+	     next;
+	  } else {
+	    # We're at the return point, or moving even furtehr away...
+	    if ($recdata+0 < $OriginalPos) {
+	       # Error exit, carp or whatever. Probably should do the
+	       # nice exit Jules does above. For now, warn and use
+	       # OriginalPos
+	       MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
+	       seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	       $OriginalPos = 0; # We're home again... Reset to prepare for
+			         # any mew warps
+	       @npos = ();
+	    } else {
+	      seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
+	      if ($recdata+0 < $npos[$#npos-1]) {
+	         if ($recdata+0 >= $npos[0]) {
+		    MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		    $ErrorFound = 1;
+		    $OriginalPos = 0; # Reset to not fool next segment loop...
+		    last;
+	         }
+	         $OriginalPos = 0; # We're home again... Reset to prepare for
+		                   # any mew warps
+	         @npos = ();
+	      }
+	    }
+	  }
+        }
+      }
+      #print STDERR "\n\nErrorFound=$ErrorFound and rectype=$rectype\n\n";
+      # Found errors above, means the file isn't complete... remove it from the batch and return immediately.
+      if ($ErrorFound or $rectype ne 'X') {
+        #MailScanner::Log::WarnLog("No end-of-message record found in %s, " .
+        #                          "retrying", $message->{id});
+        # JKF 5/12/2005 This could fail with an unblessed reference error
+        # JKF 5/12/2005 so do it by hand.
+        # JKF 5/12/2005 $message->DropFromBatch();
+        $message->{deleted} = 1;
+        $message->{gonefromdisk} = 1; # Don't try to delete the original
+        $message->{store}->Unlock(); # Unlock it so other processes can pick it up
+
+        return 0;
+      }
+    }
+    # "safety" seek, in case things go badly above. We also need to return "before" the X record, so that it is copied over below.
+    #my $CurrentPos = tell $RQf;
+    #print STDERR "MsgContSize+DataOffset = ",$MsgContSize+$DataOffset,"\nCuurentPos = ",$CurrentPos+0,"\n";
+    seek $RQf, $MsgContSize+$DataOffset, 0; # if ($MsgContSize+$DataOffset ne $CurrentPos+0);
 
     # We are now in the metadata after the message.
     #print STDERR "Reading post data\n";
@@ -485,6 +682,57 @@ sub new {
         $message->{postfixrecips}{"$recdata"} = 1;
         $TOFound = 1;
         #print STDERR "Post O Recip $recdata\n";
+      } elsif ($rectype eq 'p') {
+	# //Glenn 2007-01-16
+	# Handle p records (GOTO like in-place edit thing) by reading
+	# the pointed to data into the main message object, and
+	# silently discarding the actual p record.
+	# This p record should only point to added recipients, or
+	# moved records of the same type already handled in this
+	# segment, so lets just store the jumpoff point and loop.
+	# When we hit the next p record it should be the "jump back to
+        # original pos" one, or another forward p record... so we'll
+	# check that and act accordingly..
+	# I'm not sure this segment can have p records, but better
+	# safe than sorry.
+	next if ($recdata+0 == 0); # Ignore zero (placeholder) jumps
+	if ($recdata+0 > $MaxpRecPos) {
+	    MailScanner::Log::WarnLog("p record handling: Attempt to jump beyond end of file, aborting file.");
+	    $ErrorFound = 1;
+	    last;
+	}
+	push @npos, $recdata+0; # save jumpto position, for loop detection.
+	if ($OriginalPos == 0) {
+	   # Jump to after E record and commence
+	   $OriginalPos = tell $RQf;
+	   seek $RQf, $recdata+0, 0; # jump. we should check this works.
+	   next;
+	} else {
+	  # We're at the return point, or moving even furtehr away...
+	  if ($recdata+0 < $OriginalPos) {
+	     # Error exit, carp or whatever. Probably should do the
+	     # nice exit Jules does above. For now, warn and use
+	     # OriginalPos
+	     MailScanner::Log::WarnLog("p record handling: $recdata < $OriginalPos, using $OriginalPos instead!");
+	     seek $RQf, $OriginalPos, 0; # jump back up. we should chk this.
+	     $OriginalPos = 0; # We're home again... Reset to prepare for
+			       # any mew warps
+	     @npos = ();
+	  } else {
+	    seek $RQf, $recdata+0, 0; # jump back or forward. we should chk this works.
+	    if ($recdata+0 < $npos[$#npos-1]) {
+	       if ($recdata+0 >= $npos[0]) {
+		  MailScanner::Log::WarnLog("p record handling: Loop condition found, aborting file.");
+		  $ErrorFound = 1;
+		  # no need to reset $OriginalPos since this is the last segment.
+		  last;
+	       }
+	       $OriginalPos = 0; # We're home again... Reset to prepare for
+		                 # any mew warps
+	       @npos = ();
+	    }
+	  }
+	}
       } else {
         # Some other record type. Just store it and move on.
         push @{$message->{metadata}}, "$rectype$recdata";
@@ -516,6 +764,18 @@ sub new {
     # we have something there
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
+    # Decode ISO subject lines into UTF8
+    # Needed for UTF8 support in MailWatch 2.0
+    eval {
+     $message->{utf8subject} = Encode::decode('MIME-Header',
+                                              $message->{subject});
+    };
+    if ($@) {
+     # Eval failed - store a copy of the subject before MIME::WordDecoder
+     # is run, as this appears to destroy the characters of some subjects
+     $message->{utf8subject} = $message->{subject};
+    }
+
     # Decode the ISO encoded Subject line
     # Over-ride the default default character set handler so it does it
     # much better than the MIME-tools default handling.
@@ -923,7 +1183,7 @@ sub new {
   # its incoming queue.
   # I am passed a hash of queues --> space-separated string of message ids
   sub KickMessage {
-    my($queue2ids) = @_;
+    my($queue2ids, $sendmail2) = @_;
     my($queue);
 
     # Do a kick for every queue that contains some message ids
@@ -1221,7 +1481,7 @@ sub FindHashDirDepth {
     my(%ModDate, $mta, $file, $file1, $file2, $tmpdate);
     my(@SortedFiles, $id, $newmessage, @queuedirnames);
     my($batchempty, $h1, $h2, $delay, $CriticalQueueSize);
-    my($nlinks, $headerfileumask, $invalidfiles);
+    my($nlinks, $headerfileumask, $invalidfiles, $mtime);
 
     $queuedir  = new DirHandle;
     $queue1dir = new DirHandle;
@@ -1382,11 +1642,13 @@ sub FindHashDirDepth {
 
         # In accelerated queue-clearing mode, so we don't know anything yet
         if ($UnsortedBatchesLeft>0) {
-          $nlinks = (stat $file)[3];
+          ($nlinks, $mtime) = (stat $file)[3,9];
           next if -z _; # Skip 0-length queue files
           next unless -f _;
           next unless -R _;
           next if $nlinks>1; # Files being moved into "deferred"
+        } else {
+          $mtime = $ModDate{$file};
         }
 
         # Yes I know this is a hack but it will help isolate the problem
@@ -1425,6 +1687,7 @@ sub FindHashDirDepth {
         next unless $newmessage;
 
         $batch->{messages}{"$id"} = $newmessage;
+        $newmessage->{mtime} = $mtime;
         #print STDERR "Added $id to batch\n";
         $batchempty = 0;
 
