@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SA.pm 3810 2007-01-22 12:09:57Z sysjkf $
+#   $Id: SA.pm 4349 2008-03-17 15:39:28Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ use Compress::Zlib;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3810 $, 10;
+$VERSION = substr q$Revision: 4349 $, 10;
 
 # Attributes are
 #
@@ -78,6 +78,35 @@ sub initialise {
   @SAsuccessqueue = ();
   $SAsuccessqsum  = 0;
 
+  # If the "debug-sa" command-line flag was given, then we want to print out
+  # the current time at the start of each line that is sent to STDERR,
+  # in particular the stuff from SpamAssassin.
+  if ($MailScanner::SA::Debug) {
+    my $result;
+    # Do a trial run of awk to see if it is going to work on this system.
+    eval {
+      $result = `echo 'Hello,World' | awk '{printf \"%s %s\\n\", strftime(\"%T\"), \$0}' 2>&1`;
+      #print "Result is \"$result\"\n";
+    };
+    #print "Eval result = \"$@\"\n";
+
+    # If the trial worked...
+    if ($result =~ /\d+:\d.*Hello,World/) {
+      #print STDERR "It Succeeded!\n";
+      #select STDERR; $| = 1;
+      # Re-open STDERR with the current time stuck on the front of each line.
+      # It should work okay as we have just tried it out.
+      open STDERR, "| awk '{printf \"%s %s\\n\", strftime(\"%T\"), \$0}'";
+      select STDOUT;
+    } else {
+      print STDERR "\n\n*****\n";
+      print STDERR "If 'awk' (with support for the function strftime) was\n";
+      print STDERR "available on your \$PATH then all the SpamAssassin debug\n";
+      print STDERR "output would have the current time added to the start of\n";
+      print STDERR "every line, making debugging far easier.\n*****\n\n";
+    }
+  }
+
   # Can't just do this when sendmail.pl loads, as we are still running as
   # root then & spamassassin will get confused when we are later running
   # as something else.
@@ -91,6 +120,17 @@ sub initialise {
   # So, set ENV{HOME} to desired directory, or undef it to force it to get home
   # using getpwnam of $> (EUID)
 
+  # Create the $TMPDIR for SpamAssassin if necessary, then check we can
+  # write to it. If not, change to /tmp.
+  my $tmpdir = MailScanner::Config::Value('spamassassintempdir');
+  mkdir $tmpdir;
+  stat $tmpdir; # Is the directory writeable?
+  $tmpdir = '/tmp' unless -d _ && -r _ && -w _ && -x _;
+  $ENV{'TMPDIR'} = $tmpdir;
+  MailScanner::Log::InfoLog("SpamAssassin temporary working directory is %s",
+                            $tmpdir);
+  print STDERR "SpamAssassin temp dir = $tmpdir\n";
+
   unless (MailScanner::Config::IsSimpleValue('usespamassassin') &&
           !MailScanner::Config::Value('usespamassassin')) {
     $settings{dont_copy_prefs} = 1; # Removes need for home directory
@@ -348,10 +388,12 @@ sub Checks {
   # If they never actually installed SpamAssassin, then just bail out quietly.
   return (0,0,"",0,"") unless $SpamAssassinInstalled;
 
-  my($dfhandle);
+  my($dfhandle, $SAReqHits, $HighScoreVal);
   my($dfilename, $dfile, @WholeMessage, $SAResult, $SAHitList);
   my($HighScoring, $SAScore, $maxsize, $SAReport, $GSHits);
   my $GotFromCache = undef; # Did the result come from the cache?
+  $SAReqHits = MailScanner::Config::Value('reqspamassassinscore',$message)+0.0;
+  $HighScoreVal = MailScanner::Config::Value('highspamassassinscore',$message);
   $GSHits = $message->{gshits} || 0.0;
 
   # Bail out and fake a miss if too many consecutive SA checks failed
@@ -544,6 +586,17 @@ sub Checks {
       # word on the front here or it will be put into the cache itself!
       $GotFromCache = 1;
 
+      # Need to make sure that any differences in scores are being utilized
+      if ($SAScore >= $SAReqHits) {
+        $SAResult = 1;
+        $SAHitList =~ s/required ([\d\.]+)\,/required $SAReqHits\,/;
+        $HighScoring= 1 if !$HighScoring && $SAScore >= $HighScoreVal;
+      } elsif ($SAScore < $SAReqHits) {
+        $SAResult = 0;
+        $SAHitList =~ s/required ([\d\.]+)\,/required $SAReqHits\,/;
+        $HighScoring = 0 if $HighScoring && $SAScore < $HighScoreVal;
+      }
+
       #print STDERR "Cache results are $SAResult, $HighScoring, $SAHitList, $SAScore, $SAReport\n";
       # Unlock and close the lockfile
       flock($RebuildLockH, $LOCK_UN) if $MailScanner::SA::DoingBayesRebuilds; # $BayesIsLocked;
@@ -575,7 +628,11 @@ sub Checks {
       #             "SAHitList = $SAHitList\n";
 
       # Write the record to the cache *****
-      CacheResult($md5digest, $SAResult, $HighScoring, compress($SAHitList), $SAScore, compress($SAReport));
+      # Don't cache "timed out" results.
+      if ($SAHitList ne MailScanner::Config::LanguageValue($message, 'satimedout')) {
+        CacheResult($md5digest, $SAResult, $HighScoring,
+                    compress($SAHitList), $SAScore, compress($SAReport));
+      }
 
 
       # Unlock and close the lockfile
