@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: Sendmail.pm 3825 2007-01-29 18:45:33Z sysjkf $
+#   $Id: Sendmail.pm 4212 2007-11-20 19:59:26Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -36,11 +36,12 @@ use strict 'refs';
 no  strict 'subs'; # Allow bare words for parameter %'s
 
 use DirHandle;
+use Encode;
 
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3825 $, 10;
+$VERSION = substr q$Revision: 4212 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -360,6 +361,18 @@ my($sed) = "/bin/sed";
     # So provide an IP if we haven't found one.
     $message->{clientip} = '0.0.0.0' unless $IPFound;
 
+    # Decode ISO subject lines into UTF8
+    # Needed for UTF8 support in MailWatch 2.0
+    eval {
+     $message->{utf8subject} = Encode::decode('MIME-Header',
+                                              $message->{subject});
+    };
+    if ($@) {
+     # Eval failed - store a copy of the subject before MIME::WordDecoder
+     # is run, as this appears to destroy the characters of some subjects
+     $message->{utf8subject} = $message->{subject};
+    }
+
     # Decode the ISO encoded Subject line
     #print STDERR "Before decode subject is \"" . $message->{subject} . "\"\n";
     # Over-ride the default default character set handler so it does it
@@ -690,7 +703,7 @@ my($sed) = "/bin/sed";
 
   # This now takes a hash of queues --> space-separated list of message ids
   sub KickMessage {
-    my($messages) = @_;
+    my($messages, $sendmail2) = @_;
     my @ids;
     my $args = '';
 
@@ -701,6 +714,7 @@ my($sed) = "/bin/sed";
       next unless $queue;
       # Pull off blocks of 30 messages from the current queue
       @ids = split(" ", $messages->{$queue});
+      my $sm2 = $sendmail2->{$queue};
       while(@ids) {
         @ThisBatch = splice @ids, $[, 30;
 
@@ -715,7 +729,7 @@ my($sed) = "/bin/sed";
           $args = " -OQueueDirectory=$queue " if $queue;
           $args .= $ids;
           $args .= ' &' if $background;
-          system(MailScanner::Config::Value('sendmail2') . $args);
+          system($sm2 . $args);
         }
       }
     }
@@ -820,7 +834,7 @@ my($sed) = "/bin/sed";
     my $this = shift;
     my($batch) = @_;
 
-    my($queuedirname, $queuedir, $MsgsInQueue);
+    my($queuedirname, $queuedir, $MsgsInQueue, $mtime);
     my($DirtyMsgs, $DirtyBytes, $CleanMsgs, $CleanBytes);
     my($HitLimit1, $HitLimit2, $HitLimit3, $HitLimit4);
     my($MaxCleanB, $MaxCleanM, $MaxDirtyB, $MaxDirtyM);
@@ -971,9 +985,12 @@ my($sed) = "/bin/sed";
             $HitLimit1+$HitLimit2+$HitLimit3+$HitLimit4<1) {
         # In accelerated mode, so we don't know anything about this file
         if ($UnsortedBatchesLeft>0) {
-          stat $file;
+          my @stats = stat $file;
           next unless -f _;
           next if     -z _;
+          $mtime = $stats[9];
+        } else {
+          $mtime = $ModDate{$file};
         }
 
         # must separate next two lines or $1 gets re-tainted by being part of
@@ -997,6 +1014,7 @@ my($sed) = "/bin/sed";
         next unless $newmessage;
 
         $batch->{messages}{"$id"} = $newmessage;
+        $newmessage->{mtime} = $mtime;
         $batchempty = 0;
 
         if (MailScanner::Config::Value("virusscan", $newmessage) =~ /1/ ||
