@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: SweepOther.pm,v 1.38.2.20 2005/12/17 19:04:30 jkf Exp $
+#   $Id: SweepOther.pm 4335 2008-03-12 09:51:43Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 1.38.2.20 $, 10;
+$VERSION = substr q$Revision: 4335 $, 10;
 
 # Attributes are
 #
@@ -116,7 +116,7 @@ sub ScanBatch {
 
       # X-Spanska: header ==> "Happy" virus
       if (grep /^X-Spanska:/i, @headers) {
-        MailScanner::Log::InfoLog("Other Checks: Found Happy virus in %s", $id);
+        MailScanner::Log::NoticeLog("Other Checks: Found Happy virus in %s", $id);
         $message->{otherreports}{""} .= "\"Happy\" virus\n";
         $message->{othertypes}{""}   .= "v";
         $counter++;
@@ -130,7 +130,7 @@ sub ScanBatch {
       # MIME content boundary longer than 138 ==> Eudora exploit
       if ($message->{entity} &&
           length($message->{entity}->head->multipart_boundary)>=138) {
-        MailScanner::Log::InfoLog("Other Checks: Found Eudora " .
+        MailScanner::Log::NoticeLog("Other Checks: Found Eudora " .
                                   "long-MIME-boundary exploit in %s", $id);
         $message->{otherreports}{""} .= 
           MailScanner::Config::LanguageValue($message,'eudoralongmime') . "\n";
@@ -214,8 +214,10 @@ sub ScanBatch {
 
         # Ignore if there aren't any patterns
         if ($allowexists) {
-          if ($attach =~ /$megaallow/i || $safename =~ /$megaallow/i) {
+          #print STDERR "Allow exists\n";
+          if ($attach =~ /$megaallow/is || $safename =~ /$megaallow/is) {
             $MatchFound = 1;
+            #print STDERR "Allowing filename $id\t$safename\n";
             MailScanner::Log::InfoLog("Filename Checks: Allowing %s %s",
                                       $id, $safename)
               if $LogNames;
@@ -223,13 +225,15 @@ sub ScanBatch {
         }
         # Ignore if there aren't any patterns
         if ($denyexists) {
-          if (!$MatchFound && ($attach =~ /$megadeny/i ||
-                               $safename =~ /$megadeny/i)) {
+          #print STDERR "Deny exists\n";
+          if (!$MatchFound && ($attach =~ /$megadeny/is ||
+                               $safename =~ /$megadeny/is)) {
             $MatchFound = 1;
             # It's a rejection rule, so log the error.
             $logtext = MailScanner::Config::LanguageValue($message,
                                                         'foundblockedfilename');
             $usertext = $logtext;
+            #print STDERR "Denying filetype $id\t$safename\n";
             MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
                                       $logtext, $id, $attach);
             $message->{namereports}{$safename} .= "$usertext ($safename)\n";
@@ -260,11 +264,12 @@ sub ScanBatch {
           # Skip this rule if the regexp doesn't match
           # Check both filenames, the safe and the nasty. This is for
           # TNEF messages when the nasty filename is always winmail.dat
-          next unless $attach =~ /$regexp/i || $safename =~ /$regexp/i;
+          next unless $attach =~ /$regexp/is || $safename =~ /$regexp/is;
           $MatchFound = 1;
 
           #print STDERR "\"$attach\" matched \"$regexp\" or \"$safename\" did\n";
-          if ($allowdeny =~ 'deny') {
+          if ($allowdeny =~ /^deny[^@]*$/i) {
+            # It's a deny or deny+delete, not an email address.
             # It's a rejection rule, so log the error.
             MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
                                       $logtext, $id, $attach);
@@ -274,6 +279,14 @@ sub ScanBatch {
             $message->{nameinfected}++;
             # Do we want to delete the attachment or store it?
             $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+          } elsif ($allowdeny =~ /@/) {
+          # It's email addresses so replace the recipients list.
+          $message->DeleteAllRecipients();
+          my @newto = split(" ", lc($allowdeny));
+          $global::MS->{mta}->AddRecipients($message, @newto);
+          my $newto = join(',', @newto);
+          MailScanner::Log::InfoLog("Filetype Checks: Forwarding %s %s to %s",
+                                    $id, $safename, $newto);
           } else {
             MailScanner::Log::InfoLog("Filename Checks: Allowing %s %s",
                                       $id, $safename)
@@ -389,14 +402,94 @@ sub CheckFileContentTypes {
   MailScanner::Log::WarnLog("File checker $filecommand timed out!")
     if $TimedOut;
 
+  # If we are not using "file -i" at all, then just short-circuit all
+  # of this, and check file command output only.
+  unless ($MailScanner::Config::UsingFileICommand) {
+    $Counter = CheckFileTypesRules($batch, \%FileTypes, undef);
+    return $Counter;
+  }
+
+  my(%FileiTypes);
+  $Kid  = new FileHandle;
+  $TimedOut = 0;
+
+  eval {
+    die "Can't fork: $!" unless defined($pid = open($Kid, '-|'));
+    if ($pid) {
+      # In the parent
+      local $SIG{ALRM} = sub { $TimedOut = 1; die "Command Timed Out" };
+      alarm MailScanner::Config::Value('filetimeout');
+      # Only process the output if we are scanning, not disinfecting
+      while(<$Kid>) {
+        chomp;
+        $FileiTypes{$1}{$2} = $3 if /^([^\/]+)\/([^:]+):\s*([^;]*)(;.*)?$/;
+        #print STDERR "Processing line \"$_\"\n";
+      }
+      close $Kid;
+      $PipeReturn = $?;
+      $pid = 0; # 2.54
+      alarm 0;
+      # Workaround for bug in perl shipped with Solaris 9,
+      # it doesn't unblock the SIGALRM after handling it.
+      eval {
+        my $unblockset = POSIX::SigSet->new(SIGALRM);
+        sigprocmask(SIG_UNBLOCK, $unblockset)
+          or die "Could not unblock alarm: $!\n";
+      };
+    } else {
+      # In the child
+      POSIX::setsid();
+      @filelist = <*/*>;
+      exit 0 unless @filelist;
+      #exec "$filecommand */*"; # Shouldn't do this like this!
+      exec(split(/ +/, $filecommand), '-i', @filelist);
+      MailScanner::Log::WarnLog("Can't run 'file -i' command " .
+                                "(\"$filecommand -i\"): $!");
+      exit 1;
+    }
+  };
+  alarm 0; # 2.53
+
+  # Note to self: I only close the KID in the parent, not in the child.
+  MailScanner::Log::DebugLog("Completed checking by $filecommand -i");
+
+  # Catch failures other than the alarm
+  MailScanner::Log::DieLog("File mime-type checker failed with real error: $@")
+    if $@ and $@ !~ /Command Timed Out/;
+
+  #print STDERR "pid = $pid and \@ = $@\n";
+
+  # In which case any failures must be the alarm
+  if ($@ or $pid>0) {
+    # Kill the running child process
+    my($i);
+    kill -15, $pid;
+    # Wait for up to 5 seconds for it to die
+    for ($i=0; $i<5; $i++) {
+      sleep 1;
+      waitpid($pid, &POSIX::WNOHANG);
+      ($pid=0),last unless kill(0, $pid);
+      kill -15, $pid;
+    }
+    # And if it didn't respond to 11 nice kills, we kill -9 it
+    if ($pid) {
+      kill -9, $pid;
+      waitpid $pid, 0; # 2.53
+    }
+  }
+
+  # Return failure if the command timed out, otherwise return success
+  MailScanner::Log::WarnLog("File checker $filecommand -i timed out!")
+    if $TimedOut;
+
   # Now check all the %FileTypes we have read in
-  $Counter = CheckFileTypesRules($batch, \%FileTypes);
+  $Counter = CheckFileTypesRules($batch, \%FileTypes, \%FileiTypes);
   return $Counter;
 }
 
 
 sub CheckFileTypesRules {
-  my($batch, $FileOutput) = @_;
+  my($batch, $FileOutput, $FileiOutput) = @_;
 
   my($id, $attachtypes, $message, $tnefname, $safename, $type, $attach);
   my $counter = 0;
@@ -422,6 +515,8 @@ sub CheckFileTypesRules {
     $denypatterns  =~ s/^\s+//;
     $allowpatterns =~ s/\s+$//; # Trim trailing space
     $denypatterns  =~ s/\s+$//;
+    #$allowpatterns = quotemeta $allowpatterns; # Be careful with
+    #$denypatterns  = quotemeta $denypatterns;  # quoting everything!
     @allowpatterns = split(" ", $allowpatterns);
     @denypatterns  = split(" ", $denypatterns);
     $allowexists   = @allowpatterns; # Don't use them if they are empty!
@@ -450,7 +545,7 @@ sub CheckFileTypesRules {
 
       # Ignore if there aren't any patterns
       if ($allowexists) {
-        if ($type =~ /$megaallow/i) {
+        if ($type =~ /$megaallow/is) {
           $MatchFound = 1;
           MailScanner::Log::InfoLog("Filetype Checks: Allowing %s %s",
                                     $id, $safename)
@@ -459,7 +554,7 @@ sub CheckFileTypesRules {
       }
       # Ignore if there aren't any patterns
       if ($denyexists) {
-        if (!$MatchFound && $type =~ /$megadeny/i) {
+        if (!$MatchFound && $type =~ /$megadeny/is) {
           $MatchFound = 1;
           # It's a rejection rule, so log the error.
           $logtext = MailScanner::Config::LanguageValue($message,
@@ -477,16 +572,18 @@ sub CheckFileTypesRules {
       # Work through the attachment filetype rules,
       # using the first rule that matches.
       next unless $FiletypeRules;
-      my($allowdeny, $regexp);
+      my($allowdeny, $regexp, $iregexp);
       for ($i=0; !$MatchFound && $i<@$FiletypeRules; $i++) {
-        ($allowdeny, $regexp, $logtext, $usertext)
+        ($allowdeny, $regexp, $iregexp, $logtext, $usertext)
           = split(/\0/, $FiletypeRules->[$i]);
 
-        next unless $type =~/$regexp/i;
+        next if $regexp eq ''; # Skip this rule if there's no regexp at all
+        next unless $type =~ /$regexp/is;
         #print STDERR "Filetype match: $allowdeny $regexp $logtext $usertext\n";
 
         $MatchFound = 1;
-        if ($allowdeny =~ /deny/) {
+        if ($allowdeny =~ /^deny[^@]*$/i) {
+          # It's a deny or deny+delete, not an email address
           # It's a rejection rule, so log the error.
           MailScanner::Log::InfoLog("Filetype Checks: %s (%s %s)",
                                     $logtext, $id, $attach);
@@ -496,6 +593,14 @@ sub CheckFileTypesRules {
           $message->{nameinfected}++;
           # Do we want to delete the attachment or store it?
           $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+        } elsif ($allowdeny =~ /@/) {
+          # It's email addresses so replace the recipients list.
+          $message->DeleteAllRecipients();
+          my @newto = split(" ", lc($allowdeny));
+          $global::MS->{mta}->AddRecipients($message, @newto);
+          my $newto = join(',', @newto);
+          MailScanner::Log::InfoLog("Filetype Checks: Forwarding %s %s to %s",
+                                    $id, $safename, $newto);
         } else {
           MailScanner::Log::InfoLog("Filetype Checks: Allowing %s %s",
                                     $id, $safename)
@@ -509,6 +614,131 @@ sub CheckFileTypesRules {
     }
   }
 
+  # Short-circuit all the "file -i" testing if it hasn't been used
+  return $counter unless defined $FileiOutput;
+
+  # This is a duplicate of the code above, but with all the testing done
+  # against the "file -i" output and configuration settings.
+  #print STDERR "Doing 'file -i' output processing\n";
+  while(($id, $attachtypes) = each %$FileiOutput) {
+    next unless $id;
+    $message = $batch->{messages}{$id};
+
+    # Check the message if *any* recipient wants dangerous content scanning
+    next unless MailScanner::Config::Value('dangerscan', $message) =~ /1/;
+
+    $tnefname = $message->{entity2file}{$message->{tnefentity}};
+
+    my $LogTypes = MailScanner::Config::Value('logpermittedfilemimetypes',
+                                              $message);
+
+    # Set up patterns for simple filename real rules files
+    my($allowpatterns, $denypatterns, $allowexists, $denyexists,
+       @allowpatterns, @denypatterns, $megaallow,   $megadeny);
+    $allowpatterns = MailScanner::Config::Value('allowfilemimetypes', $message);
+    $denypatterns  = MailScanner::Config::Value('denyfilemimetypes', $message);
+    $allowpatterns =~ s/^\s+//; # Trim leading space
+    $denypatterns  =~ s/^\s+//;
+    $allowpatterns =~ s/\s+$//; # Trim trailing space
+    $denypatterns  =~ s/\s+$//;
+    #$allowpatterns = quotemeta $allowpatterns; # Be careful with
+    #$denypatterns  = quotemeta $denypatterns;  # quoting everything!
+    @allowpatterns = split(" ", $allowpatterns);
+    @denypatterns  = split(" ", $denypatterns);
+    $allowexists   = @allowpatterns; # Don't use them if they are empty!
+    $denyexists    = @denypatterns;
+    $megaallow     = '(' . join(')|(',@allowpatterns) . ')';
+    $megadeny      = '(' . join(')|(',@denypatterns) . ')';
+    #print STDERR "allowpatterns = $allowpatterns\n";
+    #print STDERR "deny          = $denypatterns\n";
+    #print STDERR "megaallow     = $megaallow\n";
+    #print STDERR "deny          = $megadeny\n";
+
+
+    my($i, $FiletypeRules);
+    $FiletypeRules = MailScanner::Config::FiletypeRulesValue($message);
+
+    while(($safename, $type) = each %$attachtypes) {
+      $attach = $message->{safefile2file}{$safename} || $tnefname;
+      next if $attach eq "" && $safename eq "";
+
+      #
+      # Implement simple all-matches rulesets for allowing and denying files
+      #
+
+      my $MatchFound = 0;
+      my($logtext, $usertext);
+
+      # Ignore if there aren't any patterns
+      if ($allowexists) {
+        if ($type =~ /$megaallow/is) {
+          $MatchFound = 1;
+          MailScanner::Log::InfoLog("Filetype Checks: Allowing %s %s",
+                                    $id, $safename)
+            if $LogTypes;
+        }
+      }
+      # Ignore if there aren't any patterns
+      if ($denyexists) {
+        if (!$MatchFound && $type =~ /$megadeny/is) {
+          $MatchFound = 1;
+          # It's a rejection rule, so log the error.
+          $logtext = MailScanner::Config::LanguageValue($message,
+                                                        'foundblockedfiletype');
+          $usertext = $logtext;
+          MailScanner::Log::InfoLog("Filename Checks: %s (%s %s)",
+                                    $logtext, $id, $attach);
+          $message->{namereports}{$safename} .= "$usertext ($safename)\n";
+          $message->{nametypes}{$safename}   .= "f";
+          $counter++;
+          $message->{nameinfected}++;
+        }
+      }
+
+      # Work through the attachment filetype rules,
+      # using the first rule that matches.
+      next unless $FiletypeRules;
+      my($allowdeny, $regexp, $iregexp);
+      for ($i=0; !$MatchFound && $i<@$FiletypeRules; $i++) {
+        ($allowdeny, $regexp, $iregexp, $logtext, $usertext)
+          = split(/\0/, $FiletypeRules->[$i]);
+
+        next if $iregexp eq ''; # Skip this rule if there's no regexp at all
+        next unless $type =~ /$iregexp/is;
+        #print STDERR "Filetype match: $allowdeny $regexp $iregexp $logtext $usertext ($type)\n";
+
+        $MatchFound = 1;
+        if ($allowdeny =~ /^deny[^@]*$/i) {
+          # It's a rejection rule, so log the error.
+          MailScanner::Log::InfoLog("Filetype Mime Checks: %s (%s %s)",
+                                    $logtext, $id, $attach);
+          $message->{namereports}{$safename} .= "$usertext ($safename)\n";
+          $message->{nametypes}{$safename}   .= "f";
+          $counter++;
+          $message->{nameinfected}++;
+          # Do we want to delete the attachment or store it?
+          $message->{deleteattach}{$safename} = 1 if $allowdeny =~ /delete/;
+        } elsif ($allowdeny =~ /@/) {
+          # It's email addresses so replace the recipients list.
+          $message->DeleteAllRecipients();
+          my @newto = split(" ", lc($allowdeny));
+          $global::MS->{mta}->AddRecipients($message, @newto);
+          my $newto = join(',', @newto);
+          MailScanner::Log::InfoLog("Filetype Checks: Forwarding %s %s to %s",
+                                    $id, $safename, $newto);
+        } else {
+          MailScanner::Log::InfoLog("Filetype Mime Checks: Allowing %s %s",
+                                    $id, $safename)
+            if $LogTypes;
+        }
+      }
+      # Log it as allowed if it didn't match any rule
+      MailScanner::Log::InfoLog("Filetype Mime Checks: Allowing %s %s " .
+                                "(no match found)", $id, $safename)
+            if $LogTypes && !$MatchFound;
+    }
+  }
+
   return $counter;
 }
 
