@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: TNEF.pm 3677 2006-07-31 21:40:25Z sysjkf $
+#   $Id: TNEF.pm 4287 2008-01-31 11:56:37Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ use POSIX qw(:signal_h setsid); # For Solaris 9 SIG bug workaround
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 3677 $, 10;
+$VERSION = substr q$Revision: 4287 $, 10;
 
 my($UseTNEFModule) = 0;
 
@@ -139,6 +139,7 @@ sub InternalDecoder {
     my $addcontents = 0;
     $addcontents = 1
       if MailScanner::Config::Value('replacetnef',$message) =~ /[12]/;
+
     my($safename, @replacements);
     foreach $_ (@{$tnef->attachments}) {
       #print STDERR "Doing attachment $_\n";
@@ -174,7 +175,7 @@ sub InternalDecoder {
           $message->{entity}->attach(Type => "application/octet-stream",
                                      Encoding => "base64",
                                      Disposition => "attachment",
-                                     Filename => $safename,
+                                     Filename => $_->longname,
                                      Path => $filename);
           $message->{bodymodified} = 1;
           #print STDERR "Message = $message and Message ID = " . $message->{id} . "\n";
@@ -204,11 +205,22 @@ sub InternalDecoder {
 # Use the external TNEF program.
 # Return 1 on success, 0 on failure.
 # This can't setup file2parent as it doesn't know the name of the children.
+# New - Unpack it into a subdirectory so we don't have name clashes anywhere.
 sub ExternalDecoder {
   my($dir, $tnefname, $message) = @_;
 
+  # Create the subdir to unpack it into
+  my $unpackdir = "tnef.$$";
+  $unpackdir = $message->MakeNameSafe($unpackdir, $dir);
+  unless (mkdir "$dir/$unpackdir", 0777) {
+    MailScanner::Log::WarnLog("Trying to unpack %s in message %s, could not create subdirectory %s, failed to unpack TNEF message", $tnefname, $message->{id},
+                              "$dir/$unpackdir");
+    return 0;
+  }
+  chmod 0700, "$dir/$unpackdir";
+
   my $cmd = MailScanner::Config::Value('tnefexpander') .
-            " -f $dir/$tnefname -C $dir --overwrite";
+            " -f $dir/$tnefname -C $dir/$unpackdir --overwrite";
 
   my($kid);
   my($TimedOut, $PipeReturn, $pid);
@@ -277,40 +289,50 @@ sub ExternalDecoder {
     # It all worked, so now add everything back into the message.
     #print STDERR "Dir is \"$dir\" and tnefname is \"$tnefname\"\n";
 
-    return 1
-      unless MailScanner::Config::Value('replacetnef',$message) =~ /[12]/;
+    unless (MailScanner::Config::Value('replacetnef',$message) =~ /[12]/) {
+      # Just need to move all the unpacked files into the main attachments dir
+      my $dirh = new DirHandle "$dir/$unpackdir";
+      return 0 unless defined $dirh;
+      while (defined(my $unpacked = $dirh->read)) {
+        next unless -f "$dir/$unpackdir/$unpacked";
+        my $safe = $message->MakeNameSafe($unpacked, $dir);
+        $message->{file2parent}{$safe} = $tnefname;
+        rename "$dir/$unpackdir/$unpacked", "$dir/$safe";
+      }
+      rmdir "$dir/$unpackdir"; # Directory should be empty now
+      return 1;
+    }
+    #print STDERR "In TNEF External Decoder\n";
 
-    my $dirh = new DirHandle $dir;
+    my $dirh = new DirHandle "$dir/$unpackdir";
     return 0 unless defined $dirh;
     my($type, $encoding);
     $message->{entity}->make_multipart;
-    my($safename, @replacements);
-    while (defined($_ = $dirh->read)) {
-      #print STDERR "Directory entry is \"$_\" in \"$dir\"\n";
-      next unless -f "$dir/$_";
-      next if $_ eq $tnefname;
-      #next if /^msg[\d-]+\.txt$/;
-      $safename = $message->MakeNameSafe($_, $dir);
+    my($safename, @replacements, $unpacked);
+    while (defined($unpacked = $dirh->read)) {
+      #print STDERR "Directory entry is \"$unpacked\" in \"$dir/$unpackdir\"\n";
+      next unless -f "$dir/$unpackdir/$unpacked";
+      $safename = $message->MakeNameSafe($unpacked, $dir);
       if (/^msg[\d-]+\.txt$/) {
         ($type, $encoding) = ("text/plain", "8bit");
       } else {
         ($type, $encoding) = ("application/octet-stream", "base64");
-        if ($safename ne $_ && -f "$dir/$_") {
-          #print STDERR "Renaming '$dir/$_' to '$dir/$safename'\n";
-          my $dangerous = quotemeta $_;
-          rename "$dir/$dangerous", "$dir/$safename";
-        }
       }
+      #print STDERR "Renaming '$dir/$unpackdir/$unpacked' to '$dir/$safename'\n";
+      rename "$dir/$unpackdir/$unpacked", "$dir/$safename";
+      # The only file that ever existed in the message structure is the safename
+      $message->{file2parent}{$safename} = $tnefname;
       push @replacements, $safename;
       $message->{entity}->attach(Type => $type,
                                  Encoding => $encoding,
                                  Disposition => "attachment",
                                  Filename => $safename,
-                                 Path => "$dir/$_");
-      $message->{bodymodified} = 1;
-      $message->{foundtnefattachments} = 1;
+                                 Path => "$dir/$safename");
     }
+    $message->{bodymodified} = 1;
+    $message->{foundtnefattachments} = 1;
     undef $dirh;
+    rmdir "$dir/$unpackdir"; # Directory should be empty now
     #$message->{entity}->dump_skeleton();
 
     MailScanner::Log::InfoLog("Message %s added TNEF contents %s",
